@@ -235,6 +235,7 @@ comp_let_value {
   gcMARK2(c->flags, gc);
   gcMARK2(c->value, gc);
   gcMARK2(c->body, gc);
+  gcMARK2(c->names, gc);
 
  size:
   gcBYTES_TO_WORDS(sizeof(Scheme_Compiled_Let_Value));
@@ -362,6 +363,7 @@ cont_proc {
   gcMARK2(c->dw, gc);
   gcMARK2(c->prompt_tag, gc);
   gcMARK2(c->meta_continuation, gc);
+  gcMARK2(c->meta_continuation_src, gc);
   gcMARK2(c->common_dw, gc);
   gcMARK2(c->save_overflow, gc);
   gcMARK2(c->runstack_copied, gc);
@@ -765,6 +767,8 @@ thread_val {
   gcMARK2(pr->ku.k.p3, gc);
   gcMARK2(pr->ku.k.p4, gc);
   gcMARK2(pr->ku.k.p5, gc);
+
+  gcMARK2(pr->self_for_proc_chaperone, gc);
   
   gcMARK2(pr->list_stack, gc);
   
@@ -800,8 +804,8 @@ runstack_val {
   intptr_t *s = (intptr_t *)p;
  mark:
   void **a, **b;
-  a = (void **)s + 4 + s[2];
-  b = (void **)s + 4 + s[3];
+  a = (void **)s + 5 + s[2];
+  b = (void **)s + 5 + s[3];
   while (a < b) {
     gcMARK2(*a, gc);
     a++;
@@ -810,14 +814,14 @@ runstack_val {
  fixup:
   /* Zero out the part that we didn't mark, in case it becomes
      live later. */
-  a = (void **)s + 4;
-  b = (void **)s + 4 + s[2];
+  a = (void **)s + 5;
+  b = (void **)s + 5 + s[2];
   while (a < b) {
     *a = RUNSTACK_ZERO_VAL;
     a++;
   }
-  a = (void **)s + 4 + s[3];
-  b = (void **)s + 4 + (s[1] - 4);
+  a = (void **)s + 5 + s[3];
+  b = (void **)s + 5 + (s[1] - 5);
   while (a < b) {
     *a = RUNSTACK_ZERO_VAL;
     a++;
@@ -986,7 +990,7 @@ prefix_val {
  size:
   gcBYTES_TO_WORDS((sizeof(Scheme_Prefix) 
 		    + ((pf->num_slots-mzFLEX_DELTA) * sizeof(Scheme_Object *))
-                    + ((((pf->num_slots - (pf->num_stxes ? (pf->num_stxes+1) : 0)) + 31) / 32) 
+                    + ((((pf->num_slots - pf->num_stxes) + 31) / 32) 
                        * sizeof(int))));
 }
 
@@ -996,7 +1000,6 @@ resolve_prefix_val {
   gcMARK2(rp->toplevels, gc);
   gcMARK2(rp->stxes, gc);
   gcMARK2(rp->delay_info_rpair, gc);
-  gcMARK2(rp->uses_unsafe, gc);
 
  size:
   gcBYTES_TO_WORDS(sizeof(Resolve_Prefix));
@@ -1009,7 +1012,6 @@ comp_prefix_val {
   gcMARK2(cp->inline_variants, gc);
   gcMARK2(cp->unbound, gc);
   gcMARK2(cp->stxes, gc);
-  gcMARK2(cp->uses_unsafe, gc);
 
  size:
   gcBYTES_TO_WORDS(sizeof(Comp_Prefix));
@@ -1207,10 +1209,11 @@ mark_logger {
   Scheme_Logger *l = (Scheme_Logger *)p;
   gcMARK2(l->name, gc);
   gcMARK2(l->parent, gc);
-  gcMARK2(l->callback, gc);
-  gcMARK2(l->timestamp, gc);
+  gcMARK2(l->want_name_level_cache, gc);
+  gcMARK2(l->root_timestamp, gc);
   gcMARK2(l->syslog_level, gc);
   gcMARK2(l->stderr_level, gc);
+  gcMARK2(l->propagate_level, gc);
   gcMARK2(l->readers, gc);
  size:
   gcBYTES_TO_WORDS(sizeof(Scheme_Logger));
@@ -1327,6 +1330,38 @@ END sfs;
 
 /**********************************************************************/
 
+START letrec_check;
+
+mark_letrec_check_frame {
+ mark:
+  Letrec_Check_Frame *frame = (Letrec_Check_Frame *)p;
+  
+  gcMARK2(frame->def, gc);
+  gcMARK2(frame->next, gc);
+  gcMARK2(frame->ref, gc);
+  gcMARK2(frame->head, gc);
+  gcMARK2(frame->deferred_chain, gc);
+
+ size:
+  gcBYTES_TO_WORDS(sizeof(Letrec_Check_Frame));
+}
+
+mark_scheme_deferred_expr {
+ mark:
+  Scheme_Deferred_Expr *clos = (Scheme_Deferred_Expr *)p;
+  
+  gcMARK2(clos->expr, gc);
+  gcMARK2(clos->frame, gc);
+  gcMARK2(clos->chain_next, gc);
+
+ size:
+  gcBYTES_TO_WORDS(sizeof(Scheme_Deferred_Expr));
+}
+
+END letrec_check;
+
+/**********************************************************************/
+
 START optimize;
 
 mark_optimize_info {
@@ -1554,7 +1589,8 @@ place_async_channel_val {
   gcMARK2(pac->msg_chains, gc);
   gcMARK2(pac->wakeup_signal, gc);
 
-  /* mark master-allocated objects within each messages: */
+  /* mark master-allocated objects within each messages; the
+     raw pairs that form the list are embedded in each message block */
   j = pac->out;
   sz = pac->size;
   for (i = pac->count; i--; ) {
@@ -1563,7 +1599,7 @@ place_async_channel_val {
       gcMARK2(SCHEME_CAR(pr), gc);
       pr = SCHEME_CDR(pr);
     }
-    j = ((j + 1) & sz);
+    j = ((j + 1) % sz);
   }
 
  size:
@@ -1688,6 +1724,7 @@ mark_input_fd {
 
   gcMARK2(fd->buffer, gc);
   gcMARK2(fd->refcount, gc);
+  gcMARK2(fd->flush_handle, gc);
 
  size:
   gcBYTES_TO_WORDS(sizeof(Scheme_FD));
@@ -1795,6 +1832,7 @@ END print;
 
 START network;
 
+#ifdef USE_TCP
 mark_listener {
   listener_t *l = (listener_t *)p;
 
@@ -1809,7 +1847,6 @@ mark_listener {
   gcBYTES_TO_WORDS(sizeof(listener_t) + ((l->count - mzFLEX_DELTA) * sizeof(tcp_t)));
 }
 
-#ifdef USE_TCP
 mark_tcp {
  mark:
   Scheme_Tcp *tcp = (Scheme_Tcp *)p;
@@ -1839,7 +1876,8 @@ mark_udp_evt {
 
   gcMARK2(uw->udp, gc);
   gcMARK2(uw->str, gc);
-  gcMARK2(uw->dest_addr, gc);
+  gcMARK2(uw->dest_addrs, gc);
+  gcMARK2(uw->dest_addr_lens, gc);
 
  size:
   gcBYTES_TO_WORDS(sizeof(Scheme_UDP_Evt));
@@ -2020,6 +2058,17 @@ mark_thread_cell {
   gcBYTES_TO_WORDS(sizeof(Thread_Cell));
 }
 
+mark_plumber {
+ mark:
+  Scheme_Plumber *pl = (Scheme_Plumber *)p;
+ 
+  gcMARK2(pl->handles, gc);
+  gcMARK2(pl->weak_handles, gc);
+
+ size:
+  gcBYTES_TO_WORDS(sizeof(Scheme_Plumber));
+}
+
 END thread;
 
 /**********************************************************************/
@@ -2176,6 +2225,18 @@ mark_nack_guard_evt {
   gcBYTES_TO_WORDS(sizeof(Nack_Guard_Evt));
 }
 
+mark_active_replace_evt {
+ mark:
+  Active_Replace_Evt *a = (Active_Replace_Evt *)p;
+
+  gcMARK2(a->syncing, gc);
+  gcMARK2(a->wrapper, gc);
+  gcMARK2(a->orig, gc);
+
+ size:
+  gcBYTES_TO_WORDS(sizeof(Active_Replace_Evt));
+}
+
 mark_chaperone {
  mark:
   Scheme_Chaperone *px = (Scheme_Chaperone *)p;
@@ -2359,6 +2420,7 @@ mark_rename_table_set {
   gcMARK2(rns->other_phases, gc);
   gcMARK2(rns->share_marked_names, gc);
   gcMARK2(rns->set_identity, gc);
+  gcMARK2(rns->prior_contexts, gc);
   gcMARK2(rns->insp, gc);
  size:
   gcBYTES_TO_WORDS(sizeof(Module_Renames_Set));
